
// simple comparison of 3 algorithms to count bits set to 1 in long int

#include <stdio.h>
#include <stdlib.h>             // malloc(), free()
#include <limits.h>             // CHAR_BIT
#include <time.h>               // clock(), CLOCKS_PER_SEC

int count_bits(unsigned long n)
{
    unsigned c;                 // number of bits set to 1
    for (c = 0; n; c++)
        n &= n - 1;             // clear the least significant bit set
    return c;
}

int count_bits2(unsigned long n)
{
    unsigned c;                 // number of bits set to 1
    for (c = 0; n; n >>= 1)
        if (n & 1)
            ++c;
    return c;
}

int count_bits3(unsigned long n)
{
    // Source: bithacks
    // Count set bits in parallel. Works up to 256 bits unsigned long.

    // n = (n & 0x5555555555555555UL) + ((n >> 1) & 0x5555555555555555UL);
    n = n - ((n >> 1) & ~0UL / 3);
    //*** now we counted set bits in each pair of bits

    // n = (n & 0x3333...) + ((n >> 2) & 0x3333...);
    n = (n & ~0UL / 5) + ((n >> 2) & ~0UL / 5);
    //*** now we counted set bits in each nibble

    // n = (n & 0x0f0f...) + ((n >> 4) & 0x0f0f...);
    n = (n + (n >> 4)) & ~0UL / 17;
    //*** now we counted set bits in each byte

    // n = n * 0x010101...01, result is in highest byte
    n = (n * (~0UL / 255)) >> ((sizeof(n) - 1) * CHAR_BIT);

    return n;   // total count of set bits
}

#define N 100000000L
int main()
{
    unsigned long *p = malloc(N * sizeof(unsigned long));
    if (!p)
        return 1;
    srand(time(NULL));
    for (unsigned long i = 0; i < N; ++i)
        p[i] = i*rand();
    clock_t start = clock();
    unsigned long x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits(p[i]);
    double elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time1=%.3f\t", elapsedTime);
    printf("count=%ld\n", x);

    start = clock();
    x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits2(p[i]);
    elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time2=%.3f\t", elapsedTime);
    printf("count=%ld\n", x);

    start = clock();
    x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits3(p[i]);
    elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time3=%.3f\t", elapsedTime);
    printf("count=%ld\n", x);

    free(p);
    return 0;
}
